"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeFromArray = exports.getStringType = exports.replaceValue = exports.math = exports.parseVariablesFromWords = exports.splitFlags = exports.getBoolean = exports.returnValue = exports.getFromReturned = exports.returnedFromLine = exports.parseDoubleFunction = exports.parseFunction = exports.parseString = exports.parseCommands2 = exports.parseCommands = exports.modifiers = exports.$functions = exports.cmds = exports.parseVariables = exports.getFunction = exports.getVariable = exports.makeVariable = exports.getArgs = exports.sleep = exports.writeReturnErr = exports.$checkQuotes = exports.$checkBrackets = void 0;
const datatypes_js_1 = require("./datatypes.js");
const index_js_1 = require("./index.js");
const fs = require("fs");
const colors = require("colors");
const $checkBrackets = function (expr) {
    const holder = [];
    const openBrackets = ['(', '{', '['];
    const closedBrackets = [')', '}', ']'];
    for (let letter of expr) {
        if (openBrackets.includes(letter)) {
            holder.push(letter);
        }
        else if (closedBrackets.includes(letter)) {
            const openPair = openBrackets[closedBrackets.indexOf(letter)];
            if (holder[holder.length - 1] === openPair) {
                holder.splice(-1, 1);
            }
            else {
                holder.push(letter);
                break;
            }
        }
    }
    return (holder.length === 0); // return true if length is 0, otherwise false
};
exports.$checkBrackets = $checkBrackets;
const $checkQuotes = function (expr) {
    if (expr[0] == '"' && expr.slice(-1) == '"') {
        return true;
    }
    else {
        return false;
    }
};
exports.$checkQuotes = $checkQuotes;
const writeReturnErr = function (fileName, newdata, add) {
    fs.readFile(fileName, 'utf8', function (err, data) {
        /* if (err) {
            return
        } */
        let $ = newdata;
        if (add) {
            $ = data += newdata;
        }
        fs.writeFile(fileName, $, function (err) {
            if (err)
                return console.log(err);
        });
    });
};
exports.writeReturnErr = writeReturnErr;
async function sleep(sec) {
    return new Promise(resolve => setTimeout(resolve, sec * 1000));
}
exports.sleep = sleep;
const getArgs = function (cmd, limit, split) {
    let $ = cmd.split(" ", limit);
    $ = cmd.split($[split]);
    let returned = $[1].slice(1);
    return returned;
};
exports.getArgs = getArgs;
// helpers
function assignVariableAbsolute(__val) {
    if (__val.__type === "string") {
        __val.absoluteValue = __val.val.split('"')[1].split('"')[0];
    }
    else {
        __val.absoluteValue = __val.val;
    }
    return __val;
}
const makeVariable = function ($name, $value, $absoluteValue, $function, $type = "undefined", $mods = [], $file = null) {
    if ($function == "null") {
        if (exports.getVariable('val:' + $name.split(" = ")[0], $function, $file) == null) {
            if (!index_js_1.config[0].specifyVal) { // <specifyVal>    </specifyVal>
                index_js_1.variables.push({
                    name: $name.split(" = ")[0],
                    val: $value,
                    absoluteValue: $absoluteValue,
                    function: "null",
                    modifiers: $mods,
                    __type: $type,
                    file: $file
                });
            }
            else {
                index_js_1.variables.push({
                    name: 'val:' + $name.split(" = ")[0],
                    val: $value,
                    absoluteValue: $absoluteValue,
                    function: "null",
                    modifiers: $mods,
                    __type: $type,
                    file: $file
                });
            }
        }
        else {
            let __val = exports.getVariable('val:' + $name.split(" = ")[0], $function, $file);
            if (!__val.modifiers.includes('static')) {
                const assign = function () {
                    __val.val = $value;
                    __val = assignVariableAbsolute(__val);
                    __val.__type = $type;
                };
                if (!__val.modifiers.includes('type')) {
                    assign();
                }
                else {
                    if (__val.modifiers.includes('type') && $type == __val.__type) {
                        assign();
                    }
                    else {
                        index_js_1.handleCommand(`SyntaxError Cannot assign type "${$type}" to a variable of type "${__val.__type}"`, $function);
                    }
                }
            }
            else if (__val.modifiers.includes('static')) {
                index_js_1.handleCommand(`SyntaxError Attempted to reassign a static variable.`, $function);
            }
        }
    }
    else {
        if (exports.getVariable('$:' + $name.split(" = ")[0], $function, $file) == null) {
            index_js_1.variables.push({
                name: '$:' + $name.split(" = ")[0] + "__&func:" + $function,
                val: $value,
                absoluteValue: $absoluteValue,
                function: $function,
                modifiers: $mods,
                __type: $type,
                file: $file
            });
        }
        else {
            let __val = exports.getVariable('val:' + $name.split(" = ")[0], $function, $file);
            __val.val = $value;
            __val = assignVariableAbsolute(__val);
            __val.__type = $type;
        }
    }
};
exports.makeVariable = makeVariable;
const getVariable = function ($name, $function = "null", file) {
    if ($function == "null") {
        for (let variable of index_js_1.variables) {
            if (file) {
                if (variable.name == $name && variable.file == file) {
                    return variable;
                }
            }
            else {
                if (variable.name == $name) {
                    return variable;
                }
            }
        }
    }
    else {
        for (let variable of index_js_1.variables) {
            if (file) {
                if (variable.name == $name && $function == variable.function && variable.file == file) {
                    return variable;
                }
            }
            else {
                if (variable.name == $name && $function == variable.function) {
                    return variable;
                }
            }
        }
    }
};
exports.getVariable = getVariable;
const getFunction = function ($name) {
    for (let func of index_js_1.functions) {
        if (func.name == $name) {
            return func;
        }
    }
};
exports.getFunction = getFunction;
// Parsing helper functions
const parseVariables = function ($content, $calling = "null", $add = "", $absolute = false, $file) {
    $content = $content.split("# ")[0];
    const __util = function (list = $content.split(" ")) {
        for (let word of list) {
            let $ = word + $add;
            let val = exports.getVariable($, $calling, $file);
            if (val != null) {
                if (!$absolute) {
                    if (val.function == "null") {
                        $content = $content.replace(word, exports.getVariable($, $calling, $file).val);
                    }
                    else {
                        if ($calling = val.function) {
                            $content = $content.replace(word, exports.getVariable($, $calling, $file).val);
                        }
                    }
                }
                else {
                    if (val.function == "null") {
                        $content = $content.replace(word, exports.getVariable($, $calling, $file).absoluteValue);
                    }
                    else {
                        if ($calling == val.function) {
                            $content = $content.replace(word, exports.getVariable($, $calling, $file).absoluteValue);
                        }
                    }
                }
            }
        }
    };
    __util(); // default parsing list
    if (exports.parseFunction($content)) {
        __util(exports.parseFunction($content).split(" ")); // inside of functions
    }
    /* if (parseString($content)) {
        __util(parseString($content).split(" ")) // inside of strings
    } */
    return $content;
};
exports.parseVariables = parseVariables;
exports.cmds = [
    'val', 'repeat', 'func', 'run', 'cd', 'clear', 'write', 'write_add', 'rm',
    'listdir', 'mk', 'exec', 'debug', 'set', '#',
    'end'
];
exports.$functions = [
    'log', 'run'
];
exports.modifiers = [
    'static', 'filelocked', 'type'
];
const parseCommands = function ($content, $calling = "null") {
    let words = $content.split(" ");
    for (let word of words) {
        if (exports.$functions.includes(word) && exports.$checkBrackets(word)) {
            // $content = $content.replace(word, handleCommand(word.split("(")[0], $calling))
            console.log($content);
        }
    }
    return $content;
};
exports.parseCommands = parseCommands;
const parseCommands2 = function ($content, $calling = "null") {
    let words = $content.split(" ");
    for (let word of words) {
        if (exports.$checkBrackets(word.split("cmd:")[1])) {
            if (exports.parseFunction(word.split("cmd:")[1])) {
                index_js_1.handleCommand(exports.parseFunction(word.split("cmd:")[1]), $calling, "", 1);
            }
        }
    }
};
exports.parseCommands2 = parseCommands2;
const parseString = function ($content) {
    return $content.split(datatypes_js_1.Token.STRING)[1].split(datatypes_js_1.Token.STRING)[0];
};
exports.parseString = parseString;
const parseFunction = function ($content) {
    return $content.split(datatypes_js_1.Token.CLOSEFUNC)[0].split(datatypes_js_1.Token.OPENFUNC)[1];
};
exports.parseFunction = parseFunction;
const parseDoubleFunction = function ($content, start = 0) {
    let __ = $content.split(datatypes_js_1.Token.CLOSEFUNC)[start].split(datatypes_js_1.Token.OPENFUNC);
    return __[start + 1] + "(" + __[start + 2] + ")";
};
exports.parseDoubleFunction = parseDoubleFunction;
exports.returnedFromLine = [];
function getFromReturned(line) {
    for (let datapoint of exports.returnedFromLine) {
        if (datapoint.line = line) {
            return datapoint;
        }
    }
}
exports.getFromReturned = getFromReturned;
const returnValue = function ($line, $value) {
    exports.returnedFromLine.push({
        line: $line,
        return: $value
    });
};
exports.returnValue = returnValue;
const getBoolean = function ($string) {
    if ($string == "true") {
        return true;
    }
    else if ($string == "false") {
        return false;
    }
    else {
        return null;
    }
};
exports.getBoolean = getBoolean;
const splitFlags = function ($string, flag) {
    return $string.split("--" + flag);
};
exports.splitFlags = splitFlags;
const parseVariablesFromWords = function (s, callingFrom, addToVariables, file) {
    let $ = s.split(" ");
    for (const $_ of $) { // parse all variables from every word
        s.replace($_, exports.parseVariables($_, callingFrom, addToVariables, false, file));
    }
    return s;
};
exports.parseVariablesFromWords = parseVariablesFromWords;
const math = function (cmd) {
    const spaces = exports.parseVariables(cmd, "null", "", true, null).split(" ");
    for (let space of spaces) {
        if (!isNaN(parseInt(space))) {
            let index = spaces.indexOf(space);
            let operator = spaces[index + 1];
            let num1 = parseInt(spaces[index]);
            let num2 = parseInt(spaces[index + 2]);
            let supportedOperators = [
                datatypes_js_1.Token.PLUS,
                datatypes_js_1.Token.MINUS,
                datatypes_js_1.Token.AST,
                datatypes_js_1.Token.CARET,
                datatypes_js_1.Token.RSLASH,
                '_/'
            ];
            if (!isNaN(num2) && !isNaN(num1) && supportedOperators.includes(operator)) {
                if (operator == datatypes_js_1.Token.PLUS) {
                    return num1 + num2;
                }
                else if (operator == datatypes_js_1.Token.MINUS) {
                    return num1 - num2;
                }
                else if (operator == datatypes_js_1.Token.AST) {
                    return num1 * num2;
                }
                else if (operator == datatypes_js_1.Token.RSLASH) {
                    return num1 / num2;
                }
                else if (operator == datatypes_js_1.Token.CARET) {
                    return Math.pow(num1, num2);
                }
                else {
                    index_js_1.handleCommand(`SyntaxError Operator not supported/undefined.`);
                }
            }
            else if (!isNaN(num1) && supportedOperators.includes(operator)) {
                if (operator == "_/") {
                    return Math.sqrt(num1);
                }
                else {
                    index_js_1.handleCommand(`SyntaxError Operator not supported/undefined.`);
                }
            }
        }
    }
};
exports.math = math;
const replaceValue = function (cmd, toReplace, value) {
    cmd = cmd.replaceAll(toReplace, value);
    return cmd;
};
exports.replaceValue = replaceValue;
const getStringType = function ($value) {
    if (isNaN(parseInt($value))) {
        if ($value[0] == datatypes_js_1.Token.STRING) {
            return 'string';
        }
        else if ($value[0] == datatypes_js_1.Token.OPEN_BRACE) {
            if (exports.$checkBrackets($value)) {
                return 'table';
            }
            else {
                index_js_1.handleCommand("SyntaxError brackets not opened and closed properly.", "getStringType", "", 1);
            }
        }
        else {
            return 'null';
        }
    }
    else if (!isNaN(parseInt($value))) {
        return 'int';
    }
};
exports.getStringType = getStringType;
const removeFromArray = function (array, indexOf) {
    for (var i = 0; i < array.length; i++) {
        if (i === indexOf) {
            array.splice(i, 1);
        }
    }
    return array;
};
exports.removeFromArray = removeFromArray;
// exports
exports.default = {
    returnValue: // every default function
    exports.returnValue,
    // get functions
    getFromReturned,
    getArgs: exports.getArgs,
    getFunction: exports.getFunction,
    getVariable: exports.getVariable,
    getBoolean: exports.getBoolean,
    getStringType: exports.getStringType,
    // parsers
    parseString: exports.parseString,
    parseVariables: exports.parseVariables,
    parseCommands: exports.parseCommands,
    parseCommands2: exports.parseCommands2,
    parseFunction: exports.parseFunction,
    parseVariablesFromWords: exports.parseVariablesFromWords,
    parseDoubleFunction: exports.parseDoubleFunction,
    // checks
    $checkBrackets: exports.$checkBrackets,
    $checkQuotes: exports.$checkQuotes,
    // unnamed
    cmds: exports.cmds,
    $functions: exports.$functions,
    sleep,
    writeReturnErr: exports.writeReturnErr,
    makeVariable: exports.makeVariable,
    splitFlags: exports.splitFlags,
    math: exports.math,
    replaceValue: exports.replaceValue,
    removeFromArray: exports.removeFromArray,
    modifiers: exports.modifiers
};
